
#define LJ_PUSHPOS           "&f0S"
#define LJ_POPPOS            "&f1S"

#define BAR_MARK "242313132"

********************************************************
* Purpose:To print a 3 of 9 barcode
* Category:Laserjet Functions
* Section:2. LaserJet
* Copyright: 1992 SCT Associates, Inc
* Author: Steve Tyrakowski/Kevin Maher
*
* Description:
*       This function allows you to print a 3 of 9 barcode
*       in the current orientation at the current printer
*       position.
*
*       The bars are drawn using the PCL box drawing functions.
*       This routine pushes the current position before printing
*       the bar code, then pops it back afterwards so that the
*       print cursor is at the same spot that it started at.
*
*       The current printer position will be the upper left corner
*       of the bar code.
*
*       The character set supported by this 3 of 9 implementation
*       includes:
*
*           numbers     0-9
*           characters  A-Z (upper only)
*           marks   - . * $ / + %
*           and space Chr(32)
*
* Syntax:
*      BarCode(<cString>, [<nHeight>], [<nNarrow>], [<nWide>],[<lVertical>])
*             => cEscCodes
*
* Formal Arguments:
*   <cString>     string to print as barcode
*   <nHeight>     Bar Code Height in dots (300 per inch) default = 120
*   <nNarrow>     Width of narrow bar in dots (300/in) default = 4
*   <nWide>       Width of wide bar in dots (300/in) default = 10
*   <lVertical>   Print the barcode verticaly (default = .f.)
*
*   Note that although the width of the narrow and wide bars may be
*   changed, you should test the bar codes thoroughly if you change
*   the widths.  The 3 of 9 spec is fairly tolerant, but many scanners
*   are much fussier.  You should try to test the printed codes with the
*   actual scanners that will be used to read them.
*
* Returns:
*     Character string containing printer escape codes
*
* See Also:
*			LJ_GoTo
********************************************************

FUNCTION BarCode
PARAMETERS cString, nHeight, nNWidth, nWWidth, lVertical
PRIVATE cNB,  cNS, cWB, cWS
DIMENSION aBar(44)
PRIVATE nChar, nLen, nCnt, cCodes, nParms


  nParms = Parameters()
  if nParms < 5
    lVertical = .F.
  endif
  if nParms < 4
    nWWidth = 10
  endif
  if nParms < 3
    nNWidth = 4
  endif
  if nParms < 2
    nHeight = 120
  endif

  nCnt = 1
  cCodes = ""

  IF lVertical
    cNB = ("*c"  + LTrim(Str(nNWidth)) + "b" + LTrim(Str(nHeight)) + "a0P")
    cNS = ("*p+" + LTrim(Str(nNWidth)) + "Y")
    cWB = ("*c"  + LTrim(Str(nWWidth)) + "b" + LTrim(Str(nHeight)) + "a0P")
    cWS = ("*p+" + LTrim(Str(nWWidth)) + "Y")
  ELSE
    cNB = ("*c"  + LTrim(Str(nNWidth)) + "a" + LTrim(Str(nHeight)) + "b0P")
    cNS = ("*p+" + LTrim(Str(nNWidth)) + "X")
    cWB = ("*c"  + LTrim(Str(nWWidth)) + "a" + LTrim(Str(nHeight)) + "b0P")
    cWS = ("*p+" + LTrim(Str(nWWidth)) + "X")
  ENDIF

  aBar[1]  = "132423231"       && "1"
  aBar[2]  = "231423231"       && "2"
  aBar[3]  = "131423232"       && "3"
  aBar[4]  = "232413231"       && "4"
  aBar[5]  = "132413232"       && "5"
  aBar[6]  = "231413232"       && "6"
  aBar[7]  = "232423131"       && "7"
  aBar[8]  = "132423132"       && "8"
  aBar[9]  = "231423132"       && "9"
  aBar[10] = "232413132"       && "0"

  aBar[11] = "132324231"       && "A"
  aBar[12] = "231324231"       && "B"
  aBar[13] = "131324232"       && "C"
  aBar[14] = "232314231"       && "D"
  aBar[15] = "132314232"       && "E"
  aBar[16] = "231314232"       && "F"
  aBar[17] = "232324131"       && "G"
  aBar[18] = "132324132"       && "H"
  aBar[19] = "231324132"       && "I"
  aBar[20] = "232314132"       && "J"
  aBar[21] = "132323241"       && "K"
  aBar[22] = "231323241"       && "L"
  aBar[23] = "131323242"       && "M"
  aBar[24] = "232313241"       && "N"
  aBar[25] = "132313242"       && "O"
  aBar[26] = "231313242"       && "P"
  aBar[27] = "232323141"       && "Q"
  aBar[28] = "132323142"       && "R"
  aBar[29] = "231323142"       && "S"
  aBar[30] = "232313142"       && "T"
  aBar[31] = "142323231"       && "U"
  aBar[32] = "241323231"       && "V"
  aBar[33] = "141323232"       && "W"
  aBar[34] = "242313231"       && "X"
  aBar[35] = "142313232"       && "Y"
  aBar[36] = "241313232"       && "Z"
  aBar[37] = "242323131"       && "-"
  aBar[38] = "142323132"       && "."
  aBar[39] = "242313132"       && "*"
  aBar[40] = "242424232"       && "$"
  aBar[41] = "242423242"       && "/"
  aBar[42] = "242324242"       && "+"
  aBar[43] = "232424242"       && "%"
  aBar[44] = "241323132"       && " "


  cCodes = cCodes + LJ_PUSHPOS + BarChar(BAR_MARK)

  cString = Upper(cString)
  nLen = Len(cString)
	DO WHILE nCnt <= nLen
    nChar = At(Substr(cString,nCnt,1),"1234567890ABCDEFGHIJKLMNOPQRSTUVWXYZ-.*$/+%")
    cCodes = cCodes + BarChar(aBar[Iif(nChar = 0,44,nChar)])
    nCnt = nCnt + 1
	ENDDO

  cCodes = cCodes + BarChar(BAR_MARK) + LJ_POPPOS

RETURN (cCodes)
*
*-------------------------------------------------------
* This function prints a single character in barcode format
* the parameter passed is a 9 byte coded string that defines
* the Wide/Short bars/spaces that make up that character.
* To avoid lots of redefinition of string space, the 4 codes
* for widebar, widespace, narrowbar, and narrowspace were
* defined in the calling function once, and are just
* referenced here as needed.
*-------------------------------------------------------
FUNCTION BarChar
PARAMETERS cChar
PRIVATE nCnt, cType, cCodes
  nCnt = 1
  cCodes = ""
  DO WHILE nCnt < 10
    cType = Substr(cChar,nCnt,1)
    DO CASE
       CASE cType = "1"  
         cCodes = cCodes + cWB + cWS
       CASE cType = "2"  
         cCodes = cCodes + cNB + cNS
       CASE cType = "3"  
         cCodes = cCodes + cNS
       CASE cType = "4"  
         cCodes = cCodes + cWS
    ENDCASE
    nCnt = nCnt + 1
	ENDDO
RETURN (cCodes + cNS)

