
********************************************************
* Purpose:Draw a box on a laser printer
* Section:2. LaserJet
* Category:Laserjet Functions
* Copyright: 1993 SCT Associates, Inc
* Author: Steve Tyrakowski/Kevin Maher
*
* Description:
*      This function allows you to draw shaded or pattern boxes
*      using the PCL commands built-in to the LaserJet.  All dimensions
*      are to be expressed in Dots which represent 1/300th of an inch.
*
*      If a row and column are passed to the function, then the PCL cursor
*      will be positioned to that location before drawing the box.  The
*      location specified will be the upper left corner of the box.  The
*      current cursor location will be saved before drawing the box, and
*      restored afterward.
*
*
* Syntax:
*      LJ_Box(<xRow>, <xCol>, <nHeigth>, <nWidth>, [<nBorder>];
*           , [<nShade>], [<nPattern>])   => cEscCodes
*
* Formal Arguments:
*      <xRow>      row that is passed to LJ_GoTo
*
*      <xCol>      col that is passed to LJ_GoTo
*
*                  Note that row and col are both optional, in which
*                  case they will use the current location.  See the
*                  LJ_GoTo function for more info.
*
*      <nHeigth>   Heigth of box in dots (300/inch)
*
*      <nWidth>    Width of box in dots (300/inch)
*
*      <nBorder>   Thickness of lines that make up box in dots(300/inch).
*                  If not specified, Rule mode will be used
*                  which results in a black box.
*
*      <nShade>    0 - 100 percent of shading of box.  Note that there
*                  are only nine distinct levels available based on the
*                  following ranges of percentages:
*
*                  0       No shading.
*                  1  - 2
*                  3  -10
*                  11 -20
*                  21 -35
*                  36 -55
*                  56 -80
*                  81 -99
*                  100     All black.
*
*      <nPattern>  Pattern to fill box with.  There are 6 defined patterns
*                  available as follows:
*
*                  1 - Horizontal lines
*                  2 - Vertical lines
*                  3 - Diagonal lines lower left to upper right
*                  4 - Diagonal lines upper left to lower right
*                  5 - Pattern #1 and Patter #2 combined
*                  6 - Pattern #3 and Patter #4 combined
*
* Returns:
*      Character field containing Escape codes for box draw.
*
* Examples:
*      ?? LJ_Box(300,600,150,1200)
*         draws a black box .5 inches tall by 4 inches wide
*         starting 1 inch from top and two inches from left.
*
*      ?? LJ_Box(300,600,150,1200,3,20)
*         draws a box .5 inches tall by 4 inches wide
*         starting 1 inch from top and two inches from left.
*         the box has a border that is 3 dots thick (1/100th inch)
*         and is filled in with a 20% grey-scale shading.
*
*
* See Also:
*      LJ_VLine() LJ_HLine() LJ_GoTo() LJ_Rec() LJ_Run()
********************************************************

FUNCTION LJ_Box
PARAMETERS nRow, nCol, nHeight, nWidth, nBorder, nShade, nPattern
PRIVATE cReturn, cHeight, cWidth, nParmCnt, nB, nS, nP
  nParmCnt = Parameters()
  nB = Iif(nParmCnt>4,nBorder ,0 )
  nS = Iif(nParmCnt>5,nShade  ,0 )
  nP = Iif(nParmCnt>6,nPattern,0 )

  cHeight = AllTrim(Str(nHeight))
  cWidth  = AllTrim(Str(nWidth ))
  cReturn = LJ_GoTo(nRow, nCol)
  IF nB > 0
    cReturn = cReturn + "&f0S"     && Push Position
    cReturn = cReturn + LJ_VLine(nHeight, nBorder)
    cReturn = cReturn + LJ_HLine(nWidth,  nBorder)
    cReturn = cReturn + LJ_GoTo("","+"+cWidth)  + LJ_VLine(nHeight, nBorder)
    cReturn = cReturn + "&f1s0S"   && Pop and Push Position
    cReturn = cReturn + LJ_GoTo("+"+cHeight,"") + LJ_HLine(nWidth,  nBorder)
    cReturn = cReturn + "&f1S"     && Pop Position
  ENDIF

  DO CASE
    CASE nP > 0
      cReturn = cReturn + "*c" + cWidth + "a" + cHeight + "b";
              + AllTrim(Str(nP)) + "g3P"

    CASE nS > 0
      cReturn = cReturn + "*c" + cWidth + "a" + cHeight + "b";
              + AllTrim(Str(nS)) + "g2P"

    CASE nB == 0
      cReturn = cReturn + "*c" + cWidth + "a" + cHeight + "b0P"

  ENDCASE

RETURN cReturn

